/*
 PROJECT: MySensors / Orientation sensor for use with MySensors
 PROGRAMMER: AWI (use of Modern Device Motion Plug and MySensors libraries)
 DATE: march 22 2016/ last update: march 22 2016
 FILE: AWI OrientationSensor.ino
 LICENSE: Public domain

 Hardware: ATMega328p board w/ NRF24l01
	and MySensors 2.0 (Development)
	MPU9250 I2C sensor
		
Special:
	uses MPU9150 motion sensor board & library https://moderndevice.com/tag/mpu9250/
	
Summary:
	calculates Yaw, Pitch and Roll and sends in a custom V_ORIENTATION (V_TEXT) type.
	format: Yaw;Pitch;Roll in integer.

Remarks:
	Fixed node-id & communcation to other fixed node/ actuator
	Used board also contains BMP280 sensor, to be included.
	
Change log:
20150322 - Initial sketch in development branch, first experiment...
*/



// Code base from Modern Device:   https://moderndevice.com/tag/mpu9250/
/* ------------- Motion Plug Demo Sketch -------------

Code for the Modern Device Motion Plug, which uses the MPU 9250 9-axis accelerometer/gyro/compass chip from Invensense.

Lightly modified version of AvrCopter.ino from rpicopter: https://github.com/rpicopter/ArduinoMotionSensorExample
  
Note: This is a 3.3v board. If you are using a 5 volt Arduino (or compatible), 
plug the + line into 3.3v and use series resistors on SDA and SCL for voltage signal level shifting.
The standard value 10k resistors won't work with high speed I2C, but we found that 4.7k and 3.3k work fine.

Features:
  - uses FastWire and I2Cdev by Jeff Rowberg
  - DMP enabled
  - calculates and displays gyro and quaternions
  - This has been tested using Arduino 1.0.5, 1.6.1, 
    a few variants of Arduino Pro Mini boards (atmega328), the Modern Device Robot Board Rev E. (atmega2560),
    Arduino Mega (atmega2560), the BBLeo (atmega32u4), and the JeeNode SMD (atmega328).

- This library specifies the MPU9250. 
  If you have an MPU6050, MPU6500, or MPU9150,
  change #DEFINE MPU9250 to #DEFINE MPU6050,
  #DEFINE MPU6500, or #DEFINE MPU9150, respectively in inv_mpu.h.

- Default chip configuration values, from inv_mpu.h.
  Pulled out here for reference:
    test->reg_rate_div   = 0;    // 1kHz.  
    test->reg_lpf        = 1;    // 188Hz. 
    test->reg_gyro_fsr   = 0;    // 250dps. 
    test->reg_accel_fsr  = 0x18; // 16g.
*/

#include "freeram.h"
#include "mpu.h"
#include "I2Cdev.h"

//****  MySensors *****
// Enable debug prints to serial monitor
//#define MY_DEBUG 
#define MY_RADIO_NRF24									// Enable and select radio type attached
#define MY_NODE_ID 50
// #define MY_RF24_CE_PIN 7								// Ceech board, 3.3v (7,8)  (pin default 9,10)
// #define MY_RF24_CS_PIN 8
#define DESTINATION_NODE 51								// node to send position info to (default 0 = gateway)
#define DESTINATION_SENSOR 1							// sensor to send position info

// custom Orientation type, comma delimited in degrees
#define V_ORIENTATION V_TEXT							// Own (not existing in MySensors protocol) type for orientation (payload = Yaw ; Pitch ; Roll in degrees)
#define S_ORIENTATION S_INFO							// Yaw = 0 (north, -180 to 180) ; Horizontal = 0 ; 

#include <SPI.h>
#include <MySensor.h>  

// MySensors sensor
#define orientationChild 1


int ret;
void setup() {
	// MPU settings, uses fastwire for I2C
    Fastwire::setup(400,0);
    ret = mympu_open(200);
    Serial.print("MPU init: "); Serial.println(ret);
    Serial.print("Free mem: "); Serial.println(freeRam());
    delay(100);	

    Serial.println("AWI Orientation");
}

void presentation(){
// MySensors
	present(orientationChild, S_ORIENTATION, "YawPitchRoll") ;
}


// MPU variables
unsigned int c = 0; //cumulative number of successful MPU/DMP reads
unsigned int np = 0; //cumulative number of MPU/DMP reads that brought no packet back
unsigned int err_c = 0; //cumulative number of MPU/DMP reads that brought corrupted packet
unsigned int err_o = 0; //cumulative number of MPU/DMP reads that had overflow bit set


// MySensors variables
int lastRoll = 0, lastPitch = 0, lastYaw = 0 ;				// AWI: last send values, only send changes
const unsigned long updateDelay = 100UL ;					// AWI: only update every second
unsigned long lastUpdate ;									
// standard messages
MyMessage orientationMsg(orientationChild, V_ORIENTATION);	// Standard mesage for orientation


void loop() {
	char payload[30];										// AWI: temporary buffer for payload storage
	unsigned long now = millis();							// loop time
	
    ret = mympu_update();
    //errorReporting(); // turn on for debug information

	if (false){ //(!(c%25)) { // output only every 25 MPU/DMP reads
	    Serial.print("Reads: "); Serial.print(c);
		Serial.print(" Y: "); Serial.print(mympu.ypr[0]);
	    Serial.print(" P: "); Serial.print(mympu.ypr[1]);
	    Serial.print(" R: "); Serial.print(mympu.ypr[2]);
	    Serial.print("\tgy: "); Serial.print(mympu.gyro[0]);
	    Serial.print(" gp: "); Serial.print(mympu.gyro[1]);
	    Serial.print(" gr: "); Serial.println(mympu.gyro[2]);
	    //delay(100);	
    }
	//if (now > lastUpdate + updateDelay){					// use when sleep is not used (sleep also sleeps timer)
		int Yaw   = (int)mympu.ypr[0] ;
		int Pitch = (int)mympu.ypr[1] ;
		int Roll  = (int)mympu.ypr[2] ;
		if (lastYaw != Yaw || lastPitch != Pitch || lastRoll != Roll){
			sprintf(payload, "%d;%d;%d", Yaw, Pitch, Roll);
			lastYaw   = Yaw; 
			lastPitch = Pitch; 
			lastRoll  = Roll; 
			Serial.print(" payload: ") ; Serial.println(payload);
			// send(orientationMsg.set(payload)) ;  			// Send to controller
			send(orientationMsg.setSensor(DESTINATION_SENSOR).setDestination(DESTINATION_NODE).set(payload)) ;  // Send to destination node
		lastUpdate = now ; 
	}
	sleep(20) ;
}

void errorReporting(){
  if (ret != 0){
    switch (ret) {
		case 0: c++; break;
		case 1: np++; return;
		case 2: err_o++; return;
		case 3: err_c++; return; 
		default: 
			Serial.print("READ ERROR!  ");
			Serial.println(ret);
      }
      Serial.print(np);
      Serial.print("  ");
      Serial.print(err_c);
      Serial.print(" ");
      Serial.print(err_o);
      Serial.print(" ");
  }
}
